/*
CSSGI shader (Coherent Screen Space Global Illumination)
This shader requires a depth pass and a normal map pass.
*/
texture mtlSkin2;
texture TargetMap;
texture mtlSkin3;

#define NUM_SAMPLES 4

uniform sampler2D som = sampler_state
{
   texture = (mtlSkin2);
   MinFilter = linear;
   MagFilter = linear;
   MipFilter = linear;
   AddressU = Clamp;
   AddressV = Clamp;
};

uniform sampler2D normal = sampler_state
{
   texture = (TargetMap);
   MinFilter = linear;
   MagFilter = linear;
   MipFilter = linear;
   AddressU = Clamp;
   AddressV = Clamp;
};

uniform sampler2D color = sampler_state
{
   texture = (mtlSkin3);
   MinFilter = linear;
   MagFilter = linear;
   MipFilter = linear;
   AddressU = Clamp;
   AddressV = Clamp;
};
//noise producing function to eliminate banding (got it from someone elses shader):
float rand(float2 co){

        return 0.5+(frac(sin(dot(co.xy ,float2(12.9898,78.233))) * 43758.5453))*0.5;

}

float4 main(float2 inTex : TEXCOORD0):COLOR0
{    
//calculate sampling rates:
float ratex = (1.0/800.0);
float ratey = (1.0/600.0);

//initialize occlusion sum and gi color:
float sum = 0.0;
float3 fcolor = float3(0,0,0);

//far and near clip planes:
float zFar = 8.0;
float zNear = 0.3;

//get depth at current pixel:
float prof = tex2D(som, inTex.xy).x;
//scale sample number with depth:
//int samples = round(NUM_SAMPLES/0.5+prof);
int samples = NUM_SAMPLES;//8/(1+prof);
prof = zFar * zNear / (prof * (zFar - zNear) - zFar);  //linearize z sample

//obtain normal and color at current pixel:
float3 norm = normalize(float3(tex2D(normal,inTex.xy).xyz)*2.0-float3(1.0,1.0,1.0));
float3 dcolor1 = tex2D(color, inTex.xy);

int hf = samples/2;
//int hf = NUM_SAMPLES;

//calculate kernel steps:
float incx = ratex*30;//gi radius
float incy = ratey*30;

float incx2 = ratex*8;//ao radius
float incy2 = ratey*8;

//do the actual calculations:
for(int i=-hf; i < hf; i++){
      for(int j=-hf; j < hf; j++){

      if (i != 0 || j!= 0) {
 
      float2 coords = float2(i*incx,j*incy)/prof;
      float2 coords2 = float2(i*incx2,j*incy2)/prof;

      float prof2 = tex2D(som,inTex.xy+coords*rand(inTex.xy)).x;
      prof2 = zFar * zNear / (prof2 * (zFar - zNear) - zFar);  //linearize z sample

      float prof2g = tex2D(som,inTex.xy+coords2*rand(inTex.xy)).x;
      prof2g = zFar * zNear / (prof2g * (zFar - zNear) - zFar);  //linearize z sample

      float3 norm2g = normalize(float3(tex2D(normal,inTex.xy+coords2*rand(inTex.xy)).xyz)*2.0-float3(1.0,1.0,1.0)); 

      float3 dcolor2 = tex2D(color, inTex.xy+coords*rand(inTex.xy));

      //OCCLUSION:

      //calculate approximate pixel distance:
      float3 dist2 = float3(coords2,prof-prof2g);

      //calculate normal and sampling direction coherence:
      float coherence2 = dot(normalize(-coords2),normalize(float2(norm2g.xy)));

      //if there is coherence, calculate occlusion:
      if (coherence2 > 0){
          float pformfactor2 = 0.5*((1.0-dot(norm,norm2g)))/(3.1416*pow(abs(length(dist2*2)),2.0)+0.5);//el 4: depthscale
          sum += clamp(pformfactor2*0.2,0.0,1.0);//ao intensity; 
      }

      //COLOR BLEEDING:

         if (length(dcolor2)>0.3){//color threshold
           float3 norm2 = normalize(float3(tex2D(normal,inTex.xy+coords*rand(inTex.xy)).xyz)*2.0-float3(1.0,1.0,1.0)); 
           
           //calculate approximate pixel distance:
           float3 dist = float3(coords,abs(prof-prof2));

           //calculate normal and sampling direction coherence:
           float coherence = dot(normalize(-coords),normalize(float2(norm2.xy)));

           //if there is coherence, calculate bleeding:
           if (coherence > 0){
              float pformfactor = ((1.0-dot(norm,norm2)))/(3.1416*pow(abs(length(dist*2)),2.0)+0.5);//el 4: depthscale
              fcolor += dcolor2*(clamp(pformfactor,0.0,1.0));
           }
        }
      
     }
   }
}

float3 bleeding = (fcolor/samples)*0.5;
float occlusion = 1.0-(sum/samples);
//gl_FragColor = float4(float3(dcolor1*occlusion+bleeding*0.5),1.0);
float4 outColor = float4(float3(dcolor1*occlusion+bleeding*0.5),1.0);
//outColor.rgb = bleeding+occlusion*0.5;
return outColor;
}

technique t1
{
	pass p1
	{
		PixelShader = compile ps_3_0 main();
		cullmode = CCW;
	}
}