float4x4 matMtl;
float4x4 matWorld;
float4x4 matWorldViewProj;

float4x4 matEffect1;
float4x4 matEffect2;
float4x4 matEffect3;

float4 vecViewPos;
float4 vecSunDir;
float4 vecLightPos[8];
float4 vecLightColor[8];

texture TargetMap;

sampler sceneSampler = sampler_state 
{ 
   Texture = <TargetMap>; 
   MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

texture sc_map_depth_bmap;

sampler depthSampler = sampler_state 
{ 
   Texture = <sc_map_depth_bmap>; 
   MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

// ShadowDepth map sampler
texture sc_map_shadowDepthSun_bmap;

sampler ShadowMapSampler = sampler_state
{
	Texture = <sc_map_shadowDepthSun_bmap>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

Texture sc_map_shadowAlpha_bmap;

sampler ShadowAlphaSampler = sampler_state
{
	texture = <sc_map_shadowAlpha_bmap>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter=LINEAR;
	AddressU = clamp;
	AddressV = clamp;
};

texture sc_map_2Dnoise_bmap;

sampler noiseSampler = sampler_state
{
	texture = <sc_map_2Dnoise_bmap>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter=LINEAR;
};

///////////////////////HELPER FUNCTIONS///////////////////////////////////////
// calculate the light attenuation factor
float DoLightFactor(float4 Light,float3 Pos)
{
   float fac = 0.f;
   if (Light.w > 0.f) {    
      float LD = length(Light.xyz-Pos)/Light.w;
      if (LD < 1.f)
         fac = saturate(1.f - LD);

   }
   return fac; // get the distance factor
}

// calculate the light attenuation factor on the front side
float DoLightFactor(float4 Light,float3 P,float3 N)
{
	float3 D = Light.xyz-P; // ray pointing from the light to the surface
	float NdotL = dot(N,normalize(D));   // angle between surface and light ray
	
	if (NdotL >= 0.f) 
	   return saturate(NdotL*8)*DoLightFactor(Light,P);
	else
	   return 0.f;
}

float DoLightFactorN(float4 Light,float3 P,float3 N)
{
	float3 D = Light.xyz-P; // ray pointing from the light to the surface
	float NdotL = dot(N,normalize(D));   // angle between surface and light ray
	
	if (NdotL >= 0.f) 
	   return 2 * NdotL * DoLightFactor(Light,P);
	else
	   return 0.f;
}

float4 DoPointLight(float3 P, float3 N, float4 Light, float4 LightColor)
{
	return LightColor * DoLightFactorN(Light,P,N);
}

float4 DoLight(float3 P, float3 N, int i)
{
	
	if(vecLightPos[i].w < 100000) return DoPointLight(P,N,vecLightPos[i],vecLightColor[i]);
	else return 0;
}

float light_vsm_epsilon = 0.00003;
float light_shadow_bias = 0.0025;

float doVSM(float2 depthmap, float lightDepth)
{
     	//depthmap.y = (depthmap.x * depthmap.x);
     	float lit_factor = (lightDepth <= depthmap.x);
    	
    	// Variance shadow mapping
    	float E_x2 = depthmap.y;
    	float Ex_2 = depthmap.x * depthmap.x;
    	float variance = min(max(E_x2 - Ex_2, 0.0) + light_vsm_epsilon, 1.0);
    	float m_d = (depthmap.x - lightDepth);
    	float p = variance / (variance + m_d * m_d);
    	
    	// Adjust the light color based on the shadow attenuation
    	return 1-max(lit_factor, p);
}

float decode(float4 vec)
{
		const float4 bitShifts = float4(1.0/(256.0*256.0*256.0), 1.0/(256.0*256.0), 1.0/256.0, 1);

		return dot(vec.xyzw , bitShifts);
}

static const float SHADOW_EPSILON = 0.00005f;
float TexelSize = 1.0f / 1280;
float4 vecViewPort;

float PCF3x3(sampler shadowMap, float2 texC, float depth)
{    
    float2 texCoords[9];

	// Generate the tecture co-ordinates for the specified depth-map size
	// 4 3 5
	// 1 0 2
	// 7 6 8
	texCoords[0] = texC;
	texCoords[1] = texC + float2( -vecViewPort.z, 0.0f);
	texCoords[2] = texC + float2(  vecViewPort.z, 0.0f);
	texCoords[3] = texC + float2( 0.0f, -vecViewPort.z);
	texCoords[6] = texC + float2( 0.0f,  vecViewPort.z);
	texCoords[4] = texC + float2( -vecViewPort.z, -vecViewPort.z);
	texCoords[5] = texC + float2(  vecViewPort.z, -vecViewPort.z);
	texCoords[7] = texC + float2( -vecViewPort.z,  vecViewPort.z);
	texCoords[8] = texC + float2(  vecViewPort.z,  vecViewPort.z);
	
	// Sample each of them checking whether the pixel under test is shadowed or not
	float shadowCoeff = 0.0f;
	for( int i = 0; i < 9; i++ )
	{
		//is the texel shadowed?
		//shadowCoeff += (tex2D( shadowMap, texCoords[i] ).r + SHADOW_EPSILON) < depth ? 0.0f : 1.0f;
		shadowCoeff += (((decode(tex2D(ShadowMapSampler, texCoords[i]).wzyx))) + SHADOW_EPSILON) < depth ? 0.0f : 1.0f;
	}
	
	// Get the average, divide by 9
	shadowCoeff *= .1111111f;
	
	return shadowCoeff;
}

///////////////////////END OF HELPER FUNCTIONS///////////////////////////////////////





float4 mainPS(float4 inTex:TEXCOORD0):COLOR0
{
	//get scene depth
	float depth = decode(tex2D(depthSampler,inTex.xy).wzyx);
	depth = tex2D(depthSampler,inTex.xy).x;
	
	//compute world position
	float4 vp = float4(inTex.xy * float2(2, -2) - float2(1, -1), depth, 1);
   float4 v = mul(vp, matMtl);
   float3 position = v.xyz/v.w;
	
	//calculate normals from position
   float3 normal = 0;
   normal = cross(ddx(position),ddy(position));
   	
	//camera-to-surface vector
   float3 directionToCamera = normalize(vecViewPos.xyz - position);
    
   //lighting
	float3 Ln = (float3(-70,150,20)- position);
 	float attenuation = saturate(1.0f - length(Ln)/500);
   Ln = normalize(-vecSunDir);
	float3 Nb = normalize(normal);
   float3 Vn = directionToCamera;
   float3 Hn = normalize(Vn + Ln);
   float4 lighting = lit(dot(Ln,Nb),dot(Hn,Nb),15);
	
	
	//shadow mapping
	float4 ShadowMapSamplingPos = mul(float4(position,1), mul(matEffect3,matEffect1));
	float4 RealDistance = mul(float4(position,1), matEffect3).z/matEffect2[0].w;
	
	float4 OutShadow;
	OutShadow.x = ShadowMapSamplingPos.x;
	OutShadow.y = ShadowMapSamplingPos.y;
	OutShadow.z = ShadowMapSamplingPos.w;
	OutShadow.w = RealDistance.x;
	
	float2 ProjectedTexCoords;
   ProjectedTexCoords[0] = OutShadow.x/OutShadow.z/2.0f +0.5f;
   ProjectedTexCoords[1] = -OutShadow.y/OutShadow.z/2.0f +0.5f;
	
	float shadowDepth = tex2D(ShadowMapSampler, ProjectedTexCoords).x;
	
	float shadow = doVSM(float2(shadowDepth,shadowDepth*shadowDepth),RealDistance.x);
	
	shadow = (1-shadow);
   //float shadow1 = PCF3x3(ShadowMapSampler, ProjectedTexCoords, In.Shadow.w);
   
   shadow *=(1-tex2D(ShadowAlphaSampler, ProjectedTexCoords));
   shadow += tex2D(ShadowAlphaSampler, ProjectedTexCoords);
   shadow = clamp(shadow,0,1);
	//--------------
	
	
	
	
	//lighting
	normal = normalize(normal);
	float4 color = 1;
	color.rgb = clamp(dot(normal,normalize(-vecSunDir)),0,1)*shadow;
	color.rgb += clamp((1-distance(position,float3(70,150,20))/300)*dot(normal,normalize(float3(70,150,20)- position)),0,1)*float3(0,0,1);
	color.rgb += clamp((1-distance(position,float3(-70,150,20))/300)*dot(normal,normalize(float3(-70,150,20)- position)),0,1)*float3(1,0.5,0);
	color.rgb += lighting.z;
	if(depth >= 1) color.rgb = 0;
	
	
	
	//ssao + ssgi
	float finalDepth = 0;
	float newDepth = 0;
	float3 gi = 0;
	float3 orgCol = tex2D(sceneSampler,inTex.xy).xyz;
	
	float blurDepth = 0.0025;
	
	//((depth/v.w)/20000);
	//float noise = tex2D(noiseSampler,inTex.xy).x/(4000*(1-depth));
	//float noise = tex2D(noiseSampler,inTex.xy).x/(1000*(1-depth));
	//float noise = tex2D(noiseSampler,inTex.xy).x/(10000*(((depth/v.w)/20000)));
	//float noise = tex2D(noiseSampler,inTex.xy*(depth)).x/(500*(1-depth));
	float noise = (tex2D(noiseSampler,position.xyz).x/10)*(1-depth);

	newDepth = tex2D(depthSampler,inTex.xy+float2(noise,0)).x;
	if(newDepth.x < depth.x + blurDepth && newDepth.x > depth.x-blurDepth)
	{
		finalDepth += newDepth.x;
		gi += tex2D(sceneSampler,inTex.xy+float2(noise,0)).xyz;
	}
	else
	{
		finalDepth += depth.x;
		gi += orgCol;
	}
	newDepth = tex2D(depthSampler,inTex.xy+float2(-noise,0)).x;
	if(newDepth.x < depth.x + blurDepth && newDepth.x > depth.x-blurDepth)
	{
		finalDepth += newDepth.x;
		gi += tex2D(sceneSampler,inTex.xy+float2(-noise,0)).xyz;
	}
	else
	{
		finalDepth += depth.x;
		gi += orgCol;
	}
	newDepth = tex2D(depthSampler,inTex.xy+float2(0,noise)).x;
	if(newDepth.x < depth.x + blurDepth && newDepth.x > depth.x-blurDepth)
	{
		finalDepth += newDepth.x;
		gi += tex2D(sceneSampler,inTex.xy+float2(0,noise)).xyz;
	}
	else
	{
		finalDepth += depth.x;
		gi += orgCol;
	}
	newDepth = tex2D(depthSampler,inTex.xy+float2(0,-noise)).x;
	if(newDepth.x < depth.x + blurDepth && newDepth.x > depth.x-blurDepth)
	{
		finalDepth += newDepth.x;
		gi += tex2D(sceneSampler,inTex.xy+float2(0,-noise)).xyz;
	}
	else
	{
		finalDepth += depth.x;
		gi += orgCol;
	}
	
	
	/*
	noise /= 1.5;
	newDepth = tex2D(depthSampler,inTex.xy+float2(noise,noise)).x;
	if(newDepth.x < depth.x + blurDepth && newDepth.x > depth.x-blurDepth)
	{
		finalDepth += newDepth.x;
		gi += tex2D(sceneSampler,inTex.xy+float2(noise,noise)).xyz;
	}
	else
	{
		finalDepth += depth.x;
		gi += orgCol;
	}
	newDepth = tex2D(depthSampler,inTex.xy+float2(-noise,-noise)).x;
	if(newDepth.x < depth.x + blurDepth && newDepth.x > depth.x-blurDepth)
	{
		finalDepth += newDepth.x;
		gi += tex2D(sceneSampler,inTex.xy+float2(-noise,-noise)).xyz;
	}
	else
	{
		finalDepth += depth.x;
		gi += orgCol;
	}
	newDepth = tex2D(depthSampler,inTex.xy+float2(-noise,noise)).x;
	if(newDepth.x < depth.x + blurDepth && newDepth.x > depth.x-blurDepth)
	{
		finalDepth += newDepth.x;
		gi += tex2D(sceneSampler,inTex.xy+float2(-noise,noise)).xyz;
	}
	else
	{
		finalDepth += depth.x;
		gi += orgCol;
	}
	newDepth = tex2D(depthSampler,inTex.xy+float2(noise,-noise)).x;
	if(newDepth.x < depth.x + blurDepth && newDepth.x > depth.x-blurDepth)
	{
		finalDepth += newDepth.x;
		gi += tex2D(sceneSampler,inTex.xy+float2(noise,-noise)).xyz;
	}
	else
	{
		finalDepth += depth.x;
		gi += orgCol;
	}
	*/
	color.rgb = tex2D(sceneSampler,inTex.xy);
	
	
	finalDepth /= 4;
	//finalDepth -= depth;
	
	//if(finalDepth > -0.0000095) finalDepth = 1;
	finalDepth = pow(1-distance(depth,finalDepth),1000);
		
	//finalDepth = 1-finalDepth;
	//finalDepth = pow(finalDepth,50000);
	color.rgb = finalDepth;
	//color.rgb *= min(color,gi/4);
	//color.rgb = gi/4;
	
	
	return color;
}

technique t1
{
	pass p0
	{
		//zWriteEnable = true;
		//alphaTestEnable = true;
		PixelShader = compile ps_2_a mainPS();
	}
}