float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matViewInv;
//float4x4 matMtl;

texture TargetMap; //original Scene
texture entSkin1; //entity tex

texture sc_map_shadowPoint1_bmap;
texture sc_map_shadowPoint2_bmap;
texture sc_map_shadowPoint3_bmap;

texture sc_map_depth_bmap;

float fAmbient;
float4 vecSkill1;
float4 vecSkill5;
float4 vecSkill9;
float4 vecSkill13;
float4 vecLightPos[8];
float4 vecLightColor[8];

sampler colorSampler = sampler_state 
{ 
   Texture = <entSkin1>; 
   MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler sceneSampler = sampler_state 
{ 
   Texture = <TargetMap>; 
   MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = clamp;
	AddressV = clamp;
};

sampler depthSampler = sampler_state 
{ 
   Texture = <sc_map_shadowPoint1_bmap>; 
   MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler depth2Sampler = sampler_state 
{ 
   Texture = <sc_map_shadowPoint2_bmap>; 
   MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler depth3Sampler = sampler_state 
{ 
   Texture = <sc_map_shadowPoint3_bmap>; 
   MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler sceneDepthSampler = sampler_state 
{ 
   Texture = <sc_map_depth_bmap>; 
   MinFilter = point;
	MagFilter = point;
	MipFilter = point;
};

///////////////////////HELPER FUNCTIONS///////////////////////////////////////
// calculate the light attenuation factor
float DoLightFactor(float4 Light,float3 Pos)
{
   float fac = 0.f;
   if (Light.w > 0.f) {    
      float LD = length(Light.xyz-Pos)/Light.w;
      if (LD < 1.f)
         fac = saturate(1.f - LD);

   }
   return fac; // get the distance factor
}

// calculate the light attenuation factor on the front side
float DoLightFactor(float4 Light,float3 P,float3 N)
{
	float3 D = Light.xyz-P; // ray pointing from the light to the surface
	float NdotL = dot(N,normalize(D));   // angle between surface and light ray
	
	if (NdotL >= 0.f) 
	   return saturate(NdotL*8)*DoLightFactor(Light,P);
	else
	   return 0.f;
}

float DoLightFactorN(float4 Light,float3 P,float3 N)
{
	float3 D = Light.xyz-P; // ray pointing from the light to the surface
	float NdotL = dot(N,normalize(D));   // angle between surface and light ray
	
	if (NdotL >= 0.f) 
	   return 2 * NdotL * DoLightFactor(Light,P);
	else
	   return 0.f;
}

float4 DoPointLight(float3 P, float3 N, float4 Light, float4 LightColor)
{
	return LightColor * DoLightFactorN(Light,P,N);
}

float4 DoLight(float3 P, float3 N, int i)
{
	
	if(vecLightPos[i].w < 100000) return DoPointLight(P,N,vecLightPos[i],vecLightColor[i]);
	else return 0;
}

float decode(float4 vec)
{
	const float4 bitShifts = float4(1.0/(256.0*256.0*256.0), 1.0/(256.0*256.0), 1.0/256.0, 1);
	return dot(vec.xyzw , bitShifts);
}


//float light_vsm_epsilon = 0.00003;
//float light_shadow_bias = 0.0025;
float light_vsm_epsilon = 0.00003;
float light_shadow_bias = 0.0025;

float doVSM(float2 depthmap, float lightDepth)
{
     	//depthmap.y = (depthmap.x * depthmap.x);
     	float lit_factor = (lightDepth <= depthmap.x);
    	
    	// Variance shadow mapping
    	float E_x2 = depthmap.y;
    	float Ex_2 = depthmap.x * depthmap.x;
    	float variance = min(max(E_x2 - Ex_2, 0.0) + light_vsm_epsilon, 1.0);
    	float m_d = (depthmap.x - lightDepth);
    	float p = variance / (variance + m_d * m_d);
    	
    	// Adjust the light color based on the shadow attenuation
    	return 1-max(lit_factor, p);
}

static const float SHADOW_EPSILON = 0.0005f;
float4 vecViewPort;

float PCF3x3(sampler shadowMap, float4 texC, float depth)
{    
    float4 texCoords[9];

	// Generate the tecture co-ordinates for the specified depth-map size
	// 4 3 5
	// 1 0 2
	// 7 6 8
	vecViewPort.z *= 1500;
	texCoords[0] = texC;
	texCoords[1] = texC + float4( -vecViewPort.z, 0.0f,0,0);
	texCoords[2] = texC + float4(  vecViewPort.z, 0.0f,0,0);
	texCoords[3] = texC + float4( 0.0f, -vecViewPort.z,0,0);
	texCoords[6] = texC + float4( 0.0f,  vecViewPort.z,0,0);
	texCoords[4] = texC + float4( 0, 0,vecViewPort.z,0);
	texCoords[5] = texC + float4(  0, 0,-vecViewPort.z,0);
	//texCoords[4] = texC + float4( -vecViewPort.z, -vecViewPort.z,0,0);
	//texCoords[5] = texC + float4(  vecViewPort.z, -vecViewPort.z,0,0);
	//texCoords[7] = texC + float4( -vecViewPort.z,  vecViewPort.z,0,0);
	//texCoords[8] = texC + float4(  vecViewPort.z,  vecViewPort.z,0,0);
	
	// Sample each of them checking whether the pixel under test is shadowed or not
	float shadowCoeff = 0.0f;
	for( int i = 0; i < 6; i++ )
	{
		//is the texel shadowed?
		//shadowCoeff += (tex2D( shadowMap, texCoords[i] ).r + SHADOW_EPSILON) < depth ? 0.0f : 1.0f;
		shadowCoeff += (((decode(texCUBE(shadowMap, texCoords[i]).wzyx))) + SHADOW_EPSILON) < depth ? 0.0f : 1.0f;
	}
	
	// Get the average, divide by 9
	//shadowCoeff *= .1111111f;
	shadowCoeff /= 6;
	
	return shadowCoeff;
}

///////////////////////END OF HELPER FUNCTIONS///////////////////////////////////////

void mainVS(
	in float4 inPos : POSITION,
	in float3 inNormal : NORMAL,
	in float2 inTex : TEXCOORD0,
	
	out float4 outPos: POSITION,
	//out float4 outColor: COLOR0,
	out float2 outTex : TEXCOORD0,
	out float4 outProjTex : TEXCOORD1,
	out float4 outProjScene : TEXCOORD2,
	out float4 outNormal : TEXCOORD3
)
{
	outPos = mul(inPos,matWorldViewProj);
	
	outProjScene = outPos;
	
	outTex = inTex;
	
	outProjTex.xyz = mul(inPos, matWorld).xyz;
		
	outProjTex.w = 1; // not needed atm
	
	outNormal = mul(inNormal,matWorld);
	/*
	outColor = fAmbient; // Add ambient and sun light
	for (int i=0; i<8; i++)  // Add 8 dynamic lights
		outColor += DoLight(outProjTex.xyz,mul(inNormal, matWorld).xyz,i);
	*/
}

float4 mainPS(
	float4 inColor : COLOR0,
	float2 inTex : TEXCOORD0,
	float4 inProjTex : TEXCOORD1,
	float4 inProjScene : TEXCOORD2,
	float4 inNormal : TEXCOORD3
):COLOR0
{
	float3 Ln = normalize(float3(vecSkill1.x,vecSkill1.z,vecSkill1.y) - inProjTex.xyz);
   float3 Nb = normalize(inNormal.xyz);
   /*
   float3 Vn = normalize(matViewInv[3].xyz - inProjTex.xyz);
   float3 Hn = normalize(Vn + Ln);
   float4 lighting = lit(dot(Ln,Nb),dot(Hn,Nb),25);
	
	Ln = normalize(float3(vecSkill5.x,vecSkill5.z,vecSkill5.y) - inProjTex.xyz);
	Hn = normalize(Vn + Ln);
	float4 lighting2 = lit(dot(Ln,Nb),dot(Hn,Nb),25);
	
	Ln = normalize(float3(vecSkill9.x,vecSkill9.z,vecSkill9.y) - inProjTex.xyz);
	Hn = normalize(Vn + Ln);
	float4 lighting3 = lit(dot(Ln,Nb),dot(Hn,Nb),25);
	*/
	
	float diff1 = clamp(dot(Nb,-Ln),0,1);
	Ln = normalize(float3(vecSkill5.x,vecSkill5.z,vecSkill5.y) - inProjTex.xyz);
	float diff2 = clamp(dot(Nb,-Ln),0,1);
	Ln = normalize(float3(vecSkill9.x,vecSkill9.z,vecSkill9.y) - inProjTex.xyz);
	float diff3 = clamp(dot(Nb,-Ln),0,1);
	if(diff1 > 0) diff1 = 1;
	if(diff2 > 0) diff2 = 1;
	if(diff3 > 0) diff3 = 1;
	
	float4 inLightDepth = 0;
	inLightDepth.r = distance(inProjTex.xyz,float3(vecSkill1.x,vecSkill1.z,vecSkill1.y))/vecSkill1.w;
	inLightDepth.g = distance(inProjTex.xyz,float3(vecSkill5.x,vecSkill5.z,vecSkill5.y))/vecSkill5.w;
	inLightDepth.b = distance(inProjTex.xyz,float3(vecSkill9.x,vecSkill9.z,vecSkill9.y))/vecSkill9.w;
	
   float4 finalColor = 1;
  	float depth = decode(texCUBE(depthSampler, -float4(float3(vecSkill1.x,vecSkill1.z,vecSkill1.y)-inProjTex.xyz, 1.0f)).wzyx);
  	float shadow1 = 1;
  	if(depth < inLightDepth.r-0.015) shadow1 = 0;
  	
  	//lerp(depth*depth*depth,depth*depth*depth*depth,0.5)
  	//lerp(lerp(depth*depth,depth*depth*depth,0.5),lerp(depth*depth*depth,depth*depth*depth*depth,0.5) , 0.5)
  	
  	//shadow1 = 1-doVSM(float2(depth,pow(depth,1.959)), inLightDepth.r);
  
   float shadow2 = 1;
   depth = decode(texCUBE(depth2Sampler, -float4(float3(vecSkill5.x,vecSkill5.z,vecSkill5.y)-inProjTex.xyz, 1.0f)).wzyx);
   if(depth < inLightDepth.g-0.015) shadow2 = 0;
   
   //shadow2 = 1-doVSM(float2(depth,pow(depth,1.959)), inLightDepth.g);
   
   float shadow3 = 1;
   depth = decode(texCUBE(depth3Sampler, -float4(float3(vecSkill9.x,vecSkill9.z,vecSkill9.y)-inProjTex.xyz, 1.0f)).wzyx);
   if(depth < inLightDepth.b-0.015) shadow3 = 0;
   
   //shadow3 = 1-doVSM(float2(depth,pow(depth,1.959)), inLightDepth.b);
   
  	if(inLightDepth.r < 0.99) inLightDepth.r = 0;
  	if(inLightDepth.g < 0.99) inLightDepth.g = 0;
  	if(inLightDepth.b < 0.99) inLightDepth.b = 0;
  	
  	shadow1 = shadow1*(1-inLightDepth.r)+inLightDepth.r;
  	shadow2 = shadow2*(1-inLightDepth.g)+inLightDepth.g;
  	shadow3 = shadow3*(1-inLightDepth.b)+inLightDepth.b;
  	
  	float shadow = 0;
  	//float shadow = clamp(((shadow1+diff1)*(shadow2+diff2)*(shadow3+diff3)),0,1);
  	if (vecLightPos[0].w > 0 && vecLightPos[0].w < 100000) shadow = clamp(((shadow1+diff1)),0,1);
  	if (vecLightPos[1].w > 0 && vecLightPos[1].w < 100000) shadow = clamp(((shadow1+diff1)+(shadow2+diff2))/2,0,1);
  	if (vecLightPos[2].w > 0 && vecLightPos[2].w < 100000) shadow = clamp(((shadow1+diff1)+(shadow2+diff2)+(shadow3+diff3))/3,0,1);
     
  
   finalColor.rgb = shadow;
  	
  	inProjScene.xy/=inProjScene.w;
	inProjScene.xy=inProjScene.xy*0.5f*float2(1,-1)+0.5f;
  	//finalColor.rgb *= tex2D(sceneSampler,inProjScene.xy).rgb;
  	
  	finalColor.a = tex2D(colorSampler,inTex).a;
   //finalColor.rgb = depth;
   
   /*
	float3 Ln = normalize(float3(100,400,300) - inProjTex.xyz);
   float3 Nb = normalize(inNormal.xyz);
   float3 Vn = normalize(matViewInv[3].xyz - inProjTex.xyz);
   float3 Hn = normalize(Vn + Ln);
   float4 lighting = lit(dot(Ln,Nb),dot(Hn,Nb),25);
   
   
   float diff = clamp(dot(Nb,Ln*10),0,1);
   //if(diff < 1) diff = 0;
   //finalColor.rgb = clamp(shadow,0,1) + clamp((1-pow(finalColor.r,8)),0,1);//clamp(shadow+(1-clamp(finalColor.r*8,0,1)),0,1);
   finalColor.rgb = clamp(shadow + (1-diff),0,1);//(clamp(shadow,0,1)-diff);// * tex2D(sceneSampler,inProjScene.xy).rgb;
   finalColor.rgb *= tex2D(sceneSampler,inProjScene.xy).rgb;
   */
 	
 	//finalColor.rgb = 1-doVSM(float2(depth,depth*depth), inLightDepth.r);
 	//finalColor.rgb = PCF3x3(depthSampler, -float4(float3(vecSkill1.x,vecSkill1.z,vecSkill1.y)-inProjTex.xyz, 1.0f), inLightDepth.r);
 	
 	if(vecSkill13.x != 0) finalColor.rgb = 1;
 	
 	finalColor.r = tex2D(sceneDepthSampler,inProjScene.xy).r;
 	//finalColor.rgb *= tex2D(sceneSampler,inProjScene.xy).rgb;
 	
 	//finalColor.rgb = shadow3*(1-pow(inLightDepth.b+0.02,12));
 	//finalColor.rgb = inLightDepth.b;
 	//finalColor.rgb = depth;
 	
 	return finalColor;
}

technique t1
{
	pass p0
	{
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_a mainPS();
	}
}