/******************************************************************************************************
Heat-Haze-Shader by Wolfgang "BoH_Havoc" Reichardt

Entity Textures:
	Skin1 = Normalmap
	Skin2 = Colormap 

Usage:
	Uncomment/Comment the #defines to add/remove an effect.
	
******************************************************************************************************/


/***************************************TWEAKABLES*****************************************************/

float refractStrength = 2;

//Use Colormap (entSkin2) to color the entity
//#define USECOLOR

/***************************************SHADER*CODE****************************************************/

float4x4 matWorldViewProj;
float4x4 matWorld;

float4 vecSkill1;
float4 vecSkill5;
float3 shieldColor = {1,1,2};

float4 vecViewPos;
float4 vecTime;

texture entSkin1;
texture entSkin2;
texture entSkin3;

sampler normalmapSampler = sampler_state
{
	Texture = <entSkin1>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;   
	//MinFilter = POINT;
	//MagFilter = POINT;
	//MipFilter = POINT;   
	AddressU  = wrap;
	AddressV  = wrap;
	
};

sampler normalmap2Sampler = sampler_state
{
	Texture = <entSkin2>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;   
	//MinFilter = POINT;
	//MagFilter = POINT;
	//MipFilter = POINT;   
	AddressU  = wrap;
	AddressV  = wrap;
	
};

sampler sparkSampler = sampler_state
{
	Texture = <entSkin3>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;   
	//MinFilter = POINT;
	//MagFilter = POINT;
	//MipFilter = POINT;   
	AddressU  = wrap;
	AddressV  = wrap;
	
};

texture sc_map_refract_bmap;

sampler refractionSampler=sampler_state
{
	texture=<sc_map_refract_bmap>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    addressu=clamp;
    addressv=clamp;
};

void heat_VS( in float4 inPos:POSITION, //input vertex position
					in float2 inTex:TEXCOORD0, //input texcoords
				   out float4 outPos:POSITION, //ouput transformed position
				   out float2 outTex:TEXCOORD0, //output texcoords
				   out float4 outProjectedTex:TEXCOORD1, //output projected texcoords
				   out float4 outWorld:TEXCOORD2 //output world position
				 )
{
	
	outPos=mul(inPos,matWorldViewProj);
	//save the vertex information for further projected texture coordinate computation
	outProjectedTex=outPos;
	//ouput texcoords
	outTex=inTex;
	//output worldPos
	outWorld = mul(inPos,matWorld);
}


float4 heat_PS(in float2 inTex:TEXCOORD0, //input texture coords
					  in float4 inProjectedTex:TEXCOORD1, //projectedTexcoords, needed to perform reflections,refractions
					  in float4 inWorld:TEXCOORD2 //world position, needed for refraction strength
					  ):COLOR0
{
	clip(vecSkill1.x);
	
	refractStrength /= distance(inWorld.xyz,vecViewPos.xyz)/4;
	
	vecSkill1.z = vecSkill1.z*0.125;
	
	inProjectedTex.xy/=inProjectedTex.w;
	inProjectedTex.xy=inProjectedTex.xy*0.5f*float2(1,-1)+0.5f;
	float4 pixelNormal=tex2D(normalmapSampler,inTex);
	pixelNormal = lerp(tex2D(normalmap2Sampler,inTex+(vecTime.w*0.001*vecSkill1.y)),pixelNormal,0.5);
   pixelNormal.yz=pixelNormal.zy;
   pixelNormal.xyz=(pixelNormal.xyz-0.5f)*2;
   
   float4 refraction=tex2D(refractionSampler,inProjectedTex.xy-refractStrength*pixelNormal.xz);
   float4 spark = tex2D(sparkSampler,(inTex/float2(8,1))+float2(vecSkill1.z,0));
	float4 sparkR = tex2D(sparkSampler,((inTex/float2(8,1))+float2(vecSkill1.z,0))-refractStrength*pixelNormal.xz);
	
	spark.rgb += spark.rgb*vecSkill5.xyz;
	spark.rgb = pow(spark.rgb,1.7);
	sparkR.rgb += sparkR.rgb*vecSkill5.xyz;
	sparkR.rgb = pow(sparkR.rgb,1.7);
	
	spark.rgb *= spark.a;
	sparkR.rgb *= sparkR.a*4;
	
	float4 color = 1;
	color = refraction;
	//color.gb += refraction.gb;
	
	#ifdef USECOLOR
		color.rgb *= tex2D(colorSampler,inTex).xyz;
	#endif
	
	color.a = tex2D(normalmapSampler,inTex).a;
	color.rgb += (spark.rgb+sparkR.rgb)*4;
	//color.rgb = spark;
   return color;
};

technique heat
{

	pass p0
	{
		zWriteEnable = true;
		CULLMODE = none;
		VertexShader  = compile vs_2_0 heat_VS();
		PixelShader = compile ps_2_0 heat_PS();
	}
}